/* We don't want gtkdoc scanning this file, it'll get
 * confused by seeing function definitions, so mark it as: */
/*<private_header>*/

static inline void
mc_cli_add_signals_for_account (DBusGProxy *proxy)
{
  dbus_g_proxy_add_signal (proxy, "Removed",
      G_TYPE_INVALID);
  dbus_g_proxy_add_signal (proxy, "AccountPropertyChanged",
      (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)),
      G_TYPE_INVALID);
}


/**
 * mc_cli_account_signal_callback_removed:
 * @proxy: The proxy on which mc_cli_account_connect_to_removed ()
 *  was called
 * @user_data: User-supplied data
 * @weak_object: User-supplied weakly referenced object
 *
 * Represents the signature of a callback for the signal Removed.
 */
static void
_mc_cli_account_invoke_callback_for_removed (TpProxy *tpproxy,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  mc_cli_account_signal_callback_removed callback =
      (mc_cli_account_signal_callback_removed) generic_callback;

  if (callback != NULL)
    callback (g_object_ref (tpproxy),
      user_data,
      weak_object);

  if (args != NULL)
    g_value_array_free (args);

  g_object_unref (tpproxy);
}
/**
 * mc_cli_account_connect_to_removed:
 * @proxy: A #TpProxy or subclass
 * @callback: Callback to be called when the signal is
 *   received
 * @user_data: User-supplied data for the callback
 * @destroy: Destructor for the user-supplied data, which
 *   will be called when this signal is disconnected, or
 *   before this function returns %NULL
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 * @error: If not %NULL, used to raise an error if %NULL is
 *   returned
 *
 * Connect a handler to the signal Removed.
 *
 * <![CDATA[This account has been removed.          <tp:rationale>           This is redundant with AccountRemoved, but it's still worth having,           to avoid having to bind to AccountManager.AccountRemoved to tell           you whether your Account is valid - ideally, an account-editing UI           should only care about a single Account.         </tp:rationale>]]>
 *
 * Returns: a #TpProxySignalConnection containing all of the
 * above, which can be used to disconnect the signal; or
 * %NULL if the proxy does not have the desired interface
 * or has become invalid.
 */
TpProxySignalConnection *
mc_cli_account_connect_to_removed (gpointer proxy,
    mc_cli_account_signal_callback_removed callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object,
    GError **error)
{
  GType expected_types[1] = {
      G_TYPE_INVALID };

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL, NULL);

  return tp_proxy_signal_connection_v0_new ((TpProxy *) proxy,
      MC_IFACE_QUARK_ACCOUNT, "Removed",
      expected_types,
      NULL, /* no args => no collector function */
      _mc_cli_account_invoke_callback_for_removed,
      G_CALLBACK (callback), user_data, destroy,
      weak_object, error);
}

/**
 * mc_cli_account_signal_callback_account_property_changed:
 * @proxy: The proxy on which mc_cli_account_connect_to_account_property_changed ()
 *  was called
 * @arg_Properties: <![CDATA[A map from property names in this namespace (e.g. Nickname) to           values. Properties whose values have not changed SHOULD be           omitted, but this need not be done.]]>
 * @user_data: User-supplied data
 * @weak_object: User-supplied weakly referenced object
 *
 * Represents the signature of a callback for the signal AccountPropertyChanged.
 */
static void
_mc_cli_account_collect_args_of_account_property_changed (DBusGProxy *proxy,
    GHashTable *arg_Properties,
    TpProxySignalConnection *sc)
{
  GValueArray *args = g_value_array_new (1);
  GValue blank = { 0 };
  guint i;

  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 1; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)));
  g_value_set_boxed (args->values + 0, arg_Properties);

  tp_proxy_signal_connection_v0_take_results (sc, args);
}
static void
_mc_cli_account_invoke_callback_for_account_property_changed (TpProxy *tpproxy,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  mc_cli_account_signal_callback_account_property_changed callback =
      (mc_cli_account_signal_callback_account_property_changed) generic_callback;

  if (callback != NULL)
    callback (g_object_ref (tpproxy),
      g_value_get_boxed (args->values + 0),
      user_data,
      weak_object);

  g_value_array_free (args);
  g_object_unref (tpproxy);
}
/**
 * mc_cli_account_connect_to_account_property_changed:
 * @proxy: A #TpProxy or subclass
 * @callback: Callback to be called when the signal is
 *   received
 * @user_data: User-supplied data for the callback
 * @destroy: Destructor for the user-supplied data, which
 *   will be called when this signal is disconnected, or
 *   before this function returns %NULL
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 * @error: If not %NULL, used to raise an error if %NULL is
 *   returned
 *
 * Connect a handler to the signal AccountPropertyChanged.
 *
 * <![CDATA[The values of one or more properties on this interface (that do not         specify that this signal does not apply to them) may have changed.         This does not cover properties of other interfaces, which must         provide their own change notification if appropriate.]]>
 *
 * Returns: a #TpProxySignalConnection containing all of the
 * above, which can be used to disconnect the signal; or
 * %NULL if the proxy does not have the desired interface
 * or has become invalid.
 */
TpProxySignalConnection *
mc_cli_account_connect_to_account_property_changed (gpointer proxy,
    mc_cli_account_signal_callback_account_property_changed callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object,
    GError **error)
{
  GType expected_types[2] = {
      (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)),
      G_TYPE_INVALID };

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL, NULL);

  return tp_proxy_signal_connection_v0_new ((TpProxy *) proxy,
      MC_IFACE_QUARK_ACCOUNT, "AccountPropertyChanged",
      expected_types,
      G_CALLBACK (_mc_cli_account_collect_args_of_account_property_changed),
      _mc_cli_account_invoke_callback_for_account_property_changed,
      G_CALLBACK (callback), user_data, destroy,
      weak_object, error);
}

/**
 * mc_cli_account_callback_for_remove:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a Remove method call
 * succeeds or fails.
 */
static void
_mc_cli_account_collect_callback_remove (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_mc_cli_account_invoke_callback_remove (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  mc_cli_account_callback_for_remove callback = (mc_cli_account_callback_for_remove) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * mc_cli_account_call_remove:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a Remove method call.
 *
 * <![CDATA[Delete the account.]]>
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
mc_cli_account_call_remove (gpointer proxy,
    gint timeout_ms,
    mc_cli_account_callback_for_remove callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = MC_IFACE_QUARK_ACCOUNT;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "Remove",
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "Remove", iface,
          _mc_cli_account_invoke_callback_remove,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "Remove",
              _mc_cli_account_collect_callback_remove,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _mc_cli_account_run_state_remove;
static void
_mc_cli_account_finish_running_remove (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback unused,
    gpointer user_data,
    GObject *unused2)
{
  _mc_cli_account_run_state_remove *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * mc_cli_account_run_remove:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method Remove and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * <![CDATA[Delete the account.]]>
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
mc_cli_account_run_remove (gpointer proxy,
    gint timeout_ms,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = MC_IFACE_QUARK_ACCOUNT;
  TpProxyPendingCall *pc;
  _mc_cli_account_run_state_remove state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "Remove", iface,
      _mc_cli_account_finish_running_remove,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "Remove",
          _mc_cli_account_collect_callback_remove,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/**
 * mc_cli_account_callback_for_update_parameters:
 * @proxy: the proxy on which the call was made
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a UpdateParameters method call
 * succeeds or fails.
 */
static void
_mc_cli_account_collect_callback_update_parameters (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;

  dbus_g_proxy_end_call (proxy, call, &error,
      G_TYPE_INVALID);
  tp_proxy_pending_call_v0_take_results (user_data, error,NULL);
}
static void
_mc_cli_account_invoke_callback_update_parameters (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  mc_cli_account_callback_for_update_parameters callback = (mc_cli_account_callback_for_update_parameters) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      error, user_data, weak_object);

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * mc_cli_account_call_update_parameters:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_Set: Used to pass an 'in' argument: <![CDATA[A mapping from parameter names to their values. These parameters           should be stored for future use.]]>
 * @in_Unset: Used to pass an 'in' argument: <![CDATA[A list of the names of parameters to be removed from the set of           stored values, allowing the default values to be used.           If the given parameters were not, in fact, stored, or even if they           do not exist at all, the account manager MUST accept this without           error.]]>
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a UpdateParameters method call.
 *
 * <![CDATA[Change the value of the Parameters property. Any changes will not         take effect until the next time the account is disconnected and         reconnected.          <tp:rationale>           Migration tools that twiddle the settings of all accounts shouldn't           cause an automatic disconnect and reconnect, probably. I could be           persuaded otherwise, though. Or we could add a Reconnect() method.         </tp:rationale>]]>
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
mc_cli_account_call_update_parameters (gpointer proxy,
    gint timeout_ms,
    GHashTable *in_Set,
    const gchar **in_Unset,
    mc_cli_account_callback_for_update_parameters callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = MC_IFACE_QUARK_ACCOUNT;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "UpdateParameters",
          (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_Set,
          G_TYPE_STRV, in_Unset,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "UpdateParameters", iface,
          _mc_cli_account_invoke_callback_update_parameters,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "UpdateParameters",
              _mc_cli_account_collect_callback_update_parameters,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_Set,
              G_TYPE_STRV, in_Unset,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gboolean success:1;
    gboolean completed:1;
} _mc_cli_account_run_state_update_parameters;
static void
_mc_cli_account_finish_running_update_parameters (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback unused,
    gpointer user_data,
    GObject *unused2)
{
  _mc_cli_account_run_state_update_parameters *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (args != NULL)
    g_value_array_free (args);
}

/**
 * mc_cli_account_run_update_parameters:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_Set: Used to pass an 'in' argument: <![CDATA[A mapping from parameter names to their values. These parameters           should be stored for future use.]]>
 * @in_Unset: Used to pass an 'in' argument: <![CDATA[A list of the names of parameters to be removed from the set of           stored values, allowing the default values to be used.           If the given parameters were not, in fact, stored, or even if they           do not exist at all, the account manager MUST accept this without           error.]]>
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method UpdateParameters and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * <![CDATA[Change the value of the Parameters property. Any changes will not         take effect until the next time the account is disconnected and         reconnected.          <tp:rationale>           Migration tools that twiddle the settings of all accounts shouldn't           cause an automatic disconnect and reconnect, probably. I could be           persuaded otherwise, though. Or we could add a Reconnect() method.         </tp:rationale>]]>
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
mc_cli_account_run_update_parameters (gpointer proxy,
    gint timeout_ms,
    GHashTable *in_Set,
    const gchar **in_Unset,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = MC_IFACE_QUARK_ACCOUNT;
  TpProxyPendingCall *pc;
  _mc_cli_account_run_state_update_parameters state = {
      NULL /* loop */, error,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "UpdateParameters", iface,
      _mc_cli_account_finish_running_update_parameters,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "UpdateParameters",
          _mc_cli_account_collect_callback_update_parameters,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_Set,
              G_TYPE_STRV, in_Unset,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/*
 * mc_cli_Account_add_signals:
 * @self: the #TpProxy
 * @quark: a quark whose string value is the interface
 *   name whose signals should be added
 * @proxy: the D-Bus proxy to which to add the signals
 * @unused: not used for anything
 *
 * Tell dbus-glib that @proxy has the signatures of all
 * signals on the given interface, if it's one we
 * support.
 *
 * This function should be used as a signal handler for
 * #TpProxy::interface-added.
 */
static void
mc_cli_Account_add_signals (TpProxy *self,
    guint quark,
    DBusGProxy *proxy,
    gpointer unused)
{
  if (quark == MC_IFACE_QUARK_ACCOUNT)
    mc_cli_add_signals_for_account (proxy);
}
