/* We don't want gtkdoc scanning this file, it'll get
 * confused by seeing function definitions, so mark it as: */
/*<private_header>*/

static inline void
mc_cli_add_signals_for_account_manager (DBusGProxy *proxy)
{
  dbus_g_proxy_add_signal (proxy, "AccountRemoved",
      DBUS_TYPE_G_OBJECT_PATH,
      G_TYPE_INVALID);
  dbus_g_proxy_add_signal (proxy, "AccountValidityChanged",
      DBUS_TYPE_G_OBJECT_PATH,
      G_TYPE_BOOLEAN,
      G_TYPE_INVALID);
}


/**
 * mc_cli_account_manager_signal_callback_account_removed:
 * @proxy: The proxy on which mc_cli_account_manager_connect_to_account_removed ()
 *  was called
 * @arg_Account: <![CDATA[An Account, which must not be used any more.]]>
 * @user_data: User-supplied data
 * @weak_object: User-supplied weakly referenced object
 *
 * Represents the signature of a callback for the signal AccountRemoved.
 */
static void
_mc_cli_account_manager_collect_args_of_account_removed (DBusGProxy *proxy,
    const gchar *arg_Account,
    TpProxySignalConnection *sc)
{
  GValueArray *args = g_value_array_new (1);
  GValue blank = { 0 };
  guint i;

  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 1; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, DBUS_TYPE_G_OBJECT_PATH);
  g_value_set_boxed (args->values + 0, arg_Account);

  tp_proxy_signal_connection_v0_take_results (sc, args);
}
static void
_mc_cli_account_manager_invoke_callback_for_account_removed (TpProxy *tpproxy,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  mc_cli_account_manager_signal_callback_account_removed callback =
      (mc_cli_account_manager_signal_callback_account_removed) generic_callback;

  if (callback != NULL)
    callback (g_object_ref (tpproxy),
      g_value_get_boxed (args->values + 0),
      user_data,
      weak_object);

  g_value_array_free (args);
  g_object_unref (tpproxy);
}
/**
 * mc_cli_account_manager_connect_to_account_removed:
 * @proxy: A #TpProxy or subclass
 * @callback: Callback to be called when the signal is
 *   received
 * @user_data: User-supplied data for the callback
 * @destroy: Destructor for the user-supplied data, which
 *   will be called when this signal is disconnected, or
 *   before this function returns %NULL
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 * @error: If not %NULL, used to raise an error if %NULL is
 *   returned
 *
 * Connect a handler to the signal AccountRemoved.
 *
 * <![CDATA[The given account has been removed.          <tp:rationale>           This is effectively change notification for the valid and invalid           accounts lists.         </tp:rationale>]]>
 *
 * Returns: a #TpProxySignalConnection containing all of the
 * above, which can be used to disconnect the signal; or
 * %NULL if the proxy does not have the desired interface
 * or has become invalid.
 */
TpProxySignalConnection *
mc_cli_account_manager_connect_to_account_removed (gpointer proxy,
    mc_cli_account_manager_signal_callback_account_removed callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object,
    GError **error)
{
  GType expected_types[2] = {
      DBUS_TYPE_G_OBJECT_PATH,
      G_TYPE_INVALID };

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL, NULL);

  return tp_proxy_signal_connection_v0_new ((TpProxy *) proxy,
      MC_IFACE_QUARK_ACCOUNT_MANAGER, "AccountRemoved",
      expected_types,
      G_CALLBACK (_mc_cli_account_manager_collect_args_of_account_removed),
      _mc_cli_account_manager_invoke_callback_for_account_removed,
      G_CALLBACK (callback), user_data, destroy,
      weak_object, error);
}

/**
 * mc_cli_account_manager_signal_callback_account_validity_changed:
 * @proxy: The proxy on which mc_cli_account_manager_connect_to_account_validity_changed ()
 *  was called
 * @arg_Account: <![CDATA[An Account.]]>
 * @arg_Valid: <![CDATA[True if the account is now valid.]]>
 * @user_data: User-supplied data
 * @weak_object: User-supplied weakly referenced object
 *
 * Represents the signature of a callback for the signal AccountValidityChanged.
 */
static void
_mc_cli_account_manager_collect_args_of_account_validity_changed (DBusGProxy *proxy,
    const gchar *arg_Account,
    gboolean arg_Valid,
    TpProxySignalConnection *sc)
{
  GValueArray *args = g_value_array_new (2);
  GValue blank = { 0 };
  guint i;

  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 2; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, DBUS_TYPE_G_OBJECT_PATH);
  g_value_set_boxed (args->values + 0, arg_Account);

  g_value_unset (args->values + 1);
  g_value_init (args->values + 1, G_TYPE_BOOLEAN);
  g_value_set_boolean (args->values + 1, arg_Valid);

  tp_proxy_signal_connection_v0_take_results (sc, args);
}
static void
_mc_cli_account_manager_invoke_callback_for_account_validity_changed (TpProxy *tpproxy,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  mc_cli_account_manager_signal_callback_account_validity_changed callback =
      (mc_cli_account_manager_signal_callback_account_validity_changed) generic_callback;

  if (callback != NULL)
    callback (g_object_ref (tpproxy),
      g_value_get_boxed (args->values + 0),
      g_value_get_boolean (args->values + 1),
      user_data,
      weak_object);

  g_value_array_free (args);
  g_object_unref (tpproxy);
}
/**
 * mc_cli_account_manager_connect_to_account_validity_changed:
 * @proxy: A #TpProxy or subclass
 * @callback: Callback to be called when the signal is
 *   received
 * @user_data: User-supplied data for the callback
 * @destroy: Destructor for the user-supplied data, which
 *   will be called when this signal is disconnected, or
 *   before this function returns %NULL
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 * @error: If not %NULL, used to raise an error if %NULL is
 *   returned
 *
 * Connect a handler to the signal AccountValidityChanged.
 *
 * <![CDATA[The validity of the given account has changed. New accounts are         also indicated by this signal, as an account validity change         (usually to True) on an account that did not previously exist.          <tp:rationale>           This is effectively change notification for the valid and invalid           accounts lists.         </tp:rationale>]]>
 *
 * Returns: a #TpProxySignalConnection containing all of the
 * above, which can be used to disconnect the signal; or
 * %NULL if the proxy does not have the desired interface
 * or has become invalid.
 */
TpProxySignalConnection *
mc_cli_account_manager_connect_to_account_validity_changed (gpointer proxy,
    mc_cli_account_manager_signal_callback_account_validity_changed callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object,
    GError **error)
{
  GType expected_types[3] = {
      DBUS_TYPE_G_OBJECT_PATH,
      G_TYPE_BOOLEAN,
      G_TYPE_INVALID };

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);
  g_return_val_if_fail (callback != NULL, NULL);

  return tp_proxy_signal_connection_v0_new ((TpProxy *) proxy,
      MC_IFACE_QUARK_ACCOUNT_MANAGER, "AccountValidityChanged",
      expected_types,
      G_CALLBACK (_mc_cli_account_manager_collect_args_of_account_validity_changed),
      _mc_cli_account_manager_invoke_callback_for_account_validity_changed,
      G_CALLBACK (callback), user_data, destroy,
      weak_object, error);
}

/**
 * mc_cli_account_manager_callback_for_create_account:
 * @proxy: the proxy on which the call was made
 * @out_Account: Used to return an 'out' argument if @error is %NULL: <![CDATA[The new account.]]>
 * @error: %NULL on success, or an error on failure
 * @user_data: user-supplied data
 * @weak_object: user-supplied object
 *
 * Signature of the callback called when a CreateAccount method call
 * succeeds or fails.
 */
static void
_mc_cli_account_manager_collect_callback_create_account (DBusGProxy *proxy,
    DBusGProxyCall *call,
    gpointer user_data)
{
  GError *error = NULL;
  GValueArray *args;
  GValue blank = { 0 };
  guint i;
  gchar *out_Account;

  dbus_g_proxy_end_call (proxy, call, &error,
      DBUS_TYPE_G_OBJECT_PATH, &out_Account,
      G_TYPE_INVALID);

  if (error != NULL)
    {
      tp_proxy_pending_call_v0_take_results (user_data, error,
          NULL);
      return;
    }

  args = g_value_array_new (1);
  g_value_init (&blank, G_TYPE_INT);

  for (i = 0; i < 1; i++)
    g_value_array_append (args, &blank);

  g_value_unset (args->values + 0);
  g_value_init (args->values + 0, DBUS_TYPE_G_OBJECT_PATH);
  g_value_take_boxed (args->values + 0, out_Account);
  tp_proxy_pending_call_v0_take_results (user_data, NULL, args);
}
static void
_mc_cli_account_manager_invoke_callback_create_account (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback generic_callback,
    gpointer user_data,
    GObject *weak_object)
{
  mc_cli_account_manager_callback_for_create_account callback = (mc_cli_account_manager_callback_for_create_account) generic_callback;

  if (error != NULL)
    {
      callback ((TpProxy *) self,
          NULL,
          error, user_data, weak_object);
      g_error_free (error);
      return;
    }
  callback ((TpProxy *) self,
      g_value_get_boxed (args->values + 0),
      error, user_data, weak_object);

  g_value_array_free (args);
}

/**
 * mc_cli_account_manager_call_create_account:
 * @proxy: the #TpProxy
 * @timeout_ms: the timeout in milliseconds, or -1 to use the
 *   default
 * @in_Connection_Manager: Used to pass an 'in' argument: <![CDATA[The name of the connection manager, e.g. &quot;salut&quot;.]]>
 * @in_Protocol: Used to pass an 'in' argument: <![CDATA[The protocol, e.g. &quot;local-xmpp&quot;.]]>
 * @in_Display_Name: Used to pass an 'in' argument: <![CDATA[The initial value of the new account's DisplayName           property. The account manager MAY modify this to make it unique,           for instance by appending a number or the 'account'           parameter.]]>
 * @in_Parameters: Used to pass an 'in' argument: <![CDATA[Initial parameter values, as would be passed to           RequestConnection.]]>
 * @callback: called when the method call succeeds or fails
 * @user_data: user-supplied data passed to the callback
 * @destroy: called with the user_data as argument, after the
 *   call has succeeded, failed or been cancelled
 * @weak_object: A #GObject which will be weakly referenced; 
 *   if it is destroyed, this callback will automatically be
 *   disconnected
 *
 * Start a CreateAccount method call.
 *
 * <![CDATA[Request the creation of a new account. The account manager SHOULD NOT         allow invalid accounts to be created. Accounts created through this         API SHOULD have an empty PresetParameters property.]]>
 *
 * Returns: a #TpProxyPendingCall representing the call in
 *  progress. It is borrowed from the object, and will become
 *  invalid when the callback is called, the call is
 *  cancelled or the #TpProxy becomes invalid.
 */
TpProxyPendingCall *
mc_cli_account_manager_call_create_account (gpointer proxy,
    gint timeout_ms,
    const gchar *in_Connection_Manager,
    const gchar *in_Protocol,
    const gchar *in_Display_Name,
    GHashTable *in_Parameters,
    mc_cli_account_manager_callback_for_create_account callback,
    gpointer user_data,
    GDestroyNotify destroy,
    GObject *weak_object)
{
  GError *error = NULL;
  GQuark interface = MC_IFACE_QUARK_ACCOUNT_MANAGER;
  DBusGProxy *iface;

  g_return_val_if_fail (TP_IS_PROXY (proxy), NULL);

  iface = tp_proxy_borrow_interface_by_id (
      (TpProxy *) proxy,
      interface, &error);

  if (iface == NULL)
    {
      if (callback != NULL)
        callback (proxy,
            NULL,
            error, user_data, weak_object);
      g_error_free (error);
      return NULL;
    }

  if (callback == NULL)
    {
      dbus_g_proxy_call_no_reply (iface, "CreateAccount",
          G_TYPE_STRING, in_Connection_Manager,
          G_TYPE_STRING, in_Protocol,
          G_TYPE_STRING, in_Display_Name,
          (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_Parameters,
          G_TYPE_INVALID);
      return NULL;
    }
  else
    {
      TpProxyPendingCall *data;

      data = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
          interface, "CreateAccount", iface,
          _mc_cli_account_manager_invoke_callback_create_account,
          G_CALLBACK (callback), user_data, destroy,
          weak_object, FALSE);
      tp_proxy_pending_call_v0_take_pending_call (data,
          dbus_g_proxy_begin_call_with_timeout (iface,
              "CreateAccount",
              _mc_cli_account_manager_collect_callback_create_account,
              data,
              tp_proxy_pending_call_v0_completed,
              timeout_ms,
              G_TYPE_STRING, in_Connection_Manager,
              G_TYPE_STRING, in_Protocol,
              G_TYPE_STRING, in_Display_Name,
              (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_Parameters,
              G_TYPE_INVALID));

      return data;
    }
}

typedef struct {
    GMainLoop *loop;
    GError **error;
    gchar **out_Account;
    gboolean success:1;
    gboolean completed:1;
} _mc_cli_account_manager_run_state_create_account;
static void
_mc_cli_account_manager_finish_running_create_account (TpProxy *self,
    GError *error,
    GValueArray *args,
    GCallback unused,
    gpointer user_data,
    GObject *unused2)
{
  _mc_cli_account_manager_run_state_create_account *state = user_data;

  state->success = (error == NULL);
  state->completed = TRUE;
  g_main_loop_quit (state->loop);

  if (error != NULL)
    {
      if (state->error != NULL)
        *state->error = error;
      else
        g_error_free (error);

      return;
    }

  if (state->out_Account != NULL)
    *state->out_Account = g_value_dup_boxed (args->values + 0);

  g_value_array_free (args);
}

/**
 * mc_cli_account_manager_run_create_account:
 * @proxy: A #TpProxy or subclass
 * @timeout_ms: Timeout in milliseconds, or -1 for default
 * @in_Connection_Manager: Used to pass an 'in' argument: <![CDATA[The name of the connection manager, e.g. &quot;salut&quot;.]]>
 * @in_Protocol: Used to pass an 'in' argument: <![CDATA[The protocol, e.g. &quot;local-xmpp&quot;.]]>
 * @in_Display_Name: Used to pass an 'in' argument: <![CDATA[The initial value of the new account's DisplayName           property. The account manager MAY modify this to make it unique,           for instance by appending a number or the 'account'           parameter.]]>
 * @in_Parameters: Used to pass an 'in' argument: <![CDATA[Initial parameter values, as would be passed to           RequestConnection.]]>
 * @out_Account: Used to return an 'out' argument if %TRUE is returned: <![CDATA[The new account.]]>
 * @error: If not %NULL, used to return errors if %FALSE 
 *  is returned
 * @loop: If not %NULL, set before re-entering 
 *  the main loop, to point to a #GMainLoop 
 *  which can be used to cancel this call with 
 *  g_main_loop_quit(), causing a return of 
 *  %FALSE with @error set to %TP_DBUS_ERROR_CANCELLED
 *
 * Call the method CreateAccount and run the main loop
 * until it returns. Before calling this method, you must
 * add a reference to any borrowed objects you need to keep,
 * and generally ensure that everything is in a consistent
 * state.
 *
 * <![CDATA[Request the creation of a new account. The account manager SHOULD NOT         allow invalid accounts to be created. Accounts created through this         API SHOULD have an empty PresetParameters property.]]>
 *
 * Returns: TRUE on success, FALSE and sets @error on error
 */
gboolean
mc_cli_account_manager_run_create_account (gpointer proxy,
    gint timeout_ms,
    const gchar *in_Connection_Manager,
    const gchar *in_Protocol,
    const gchar *in_Display_Name,
    GHashTable *in_Parameters,
    gchar **out_Account,
    GError **error,
    GMainLoop **loop)
{
  DBusGProxy *iface;
  GQuark interface = MC_IFACE_QUARK_ACCOUNT_MANAGER;
  TpProxyPendingCall *pc;
  _mc_cli_account_manager_run_state_create_account state = {
      NULL /* loop */, error,
    out_Account,
      FALSE /* completed */, FALSE /* success */ };

  g_return_val_if_fail (TP_IS_PROXY (proxy), FALSE);

  iface = tp_proxy_borrow_interface_by_id
       ((TpProxy *) proxy, interface, error);

  if (iface == NULL)
    return FALSE;

  state.loop = g_main_loop_new (NULL, FALSE);

  pc = tp_proxy_pending_call_v0_new ((TpProxy *) proxy,
      interface, "CreateAccount", iface,
      _mc_cli_account_manager_finish_running_create_account,
      NULL, &state, NULL, NULL, TRUE);

  if (loop != NULL)
    *loop = state.loop;

  tp_proxy_pending_call_v0_take_pending_call (pc,
      dbus_g_proxy_begin_call_with_timeout (iface,
          "CreateAccount",
          _mc_cli_account_manager_collect_callback_create_account,
          pc,
          tp_proxy_pending_call_v0_completed,
          timeout_ms,
              G_TYPE_STRING, in_Connection_Manager,
              G_TYPE_STRING, in_Protocol,
              G_TYPE_STRING, in_Display_Name,
              (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE)), in_Parameters,
          G_TYPE_INVALID));

  if (!state.completed)
    g_main_loop_run (state.loop);

  if (!state.completed)
    tp_proxy_pending_call_cancel (pc);

  if (loop != NULL)
    *loop = NULL;

  g_main_loop_unref (state.loop);

  return state.success;
}


/*
 * mc_cli_Account_Manager_add_signals:
 * @self: the #TpProxy
 * @quark: a quark whose string value is the interface
 *   name whose signals should be added
 * @proxy: the D-Bus proxy to which to add the signals
 * @unused: not used for anything
 *
 * Tell dbus-glib that @proxy has the signatures of all
 * signals on the given interface, if it's one we
 * support.
 *
 * This function should be used as a signal handler for
 * #TpProxy::interface-added.
 */
static void
mc_cli_Account_Manager_add_signals (TpProxy *self,
    guint quark,
    DBusGProxy *proxy,
    gpointer unused)
{
  if (quark == MC_IFACE_QUARK_ACCOUNT_MANAGER)
    mc_cli_add_signals_for_account_manager (proxy);
}
